@@ -188,7 +188,7 @@ static ssize_t read_net_internal(int fd, void *buf, size_t count, int socketStat
 }
 
 static int open_net_http(char *user, char *pass, char *host, int port, char *path, int flags, off_t offset, ssize_t *length) {
-	unsigned long content_length = 0;
+	unsigned long long content_length = 0;
 	ssize_t read_ret, sink_bytes = 0;
 	ssize_t send_ret, recv_ret;
 	char httpcmd[4096], charbuf, httpreply[1024], *http_response;
@@ -260,7 +260,7 @@ static int open_net_http(char *user, char *pass, char *host, int port, char *pat
 					}
 				} else {
 					if (strncasecmp(httpreply, "Content-Length:", 15) == 0) {
-						content_length = strtoul(httpreply + 16, NULL, 10);
+						content_length = strtoull(httpreply + 16, NULL, 10);
 						if (offset != 0 && http_response_code == 206) {
 							content_length += offset;
 						}
@@ -471,6 +471,25 @@ size_t fread_net(void *ptr, size_t size, size_t nmemb, NETFILE *stream) {
 			}
 
 			stream->bufused += recv_ret;
+
+			if (stream->bufused >= (size * nmemb)) {
+				break;
+			}
+
+			/*
+			 * If there is more data to be read, but the buffer is full
+			 * try to make room, or quit trying to read data
+			 */
+			if (stream->bufused == stream->bufsize) {
+				if (stream->bufsize_s != stream->bufsize) {
+					CHECKPOINT;
+					memmove(stream->buf_s, stream->buf, stream->bufused);
+					stream->buf = stream->buf_s;
+					stream->bufsize = stream->bufsize_s;
+				} else {
+					break;
+				}
+			}
 		}
 	}
 
@@ -496,8 +515,7 @@ size_t fread_net(void *ptr, size_t size, size_t nmemb, NETFILE *stream) {
 	stream->bufused -= bytes_to_copy;
 	stream->bufsize -= bytes_to_copy;
 
-	if ((stream->bufused < size && size < (stream->bufsize_s / 2)) || stream->bufused == 0) {
-		memmove(stream->buf_s, stream->buf, stream->bufused);
+	if (stream->bufused == 0) {
 		stream->buf = stream->buf_s;
 		stream->bufsize = stream->bufsize_s;
 	}
@@ -625,7 +643,7 @@ off_t lseek_net(int filedes, off_t offset, int whence) {
 	return(-1);
 }
 
-int fseek_net(NETFILE *stream, long offset, int whence) {
+int fseeko_net(NETFILE *stream, off_t offset, int whence) {
 	static char garbage[4096];
 	ssize_t newlength;
 	size_t fread_ret, bytes_to_read;
@@ -698,7 +716,11 @@ int fseek_net(NETFILE *stream, long offset, int whence) {
 	return(0);
 }
 
-long ftell_net(NETFILE *stream) {
+int fseek_net(NETFILE *stream, long offset, int whence) {
+	return(fseek_net(stream, offset, whence));
+}
+
+off_t ftello_net(NETFILE *stream) {
 	if (!stream) {
 		return(-1);
 	}
@@ -709,7 +731,11 @@ long ftell_net(NETFILE *stream) {
 	return(stream->pos);
 }
 
-long flength_net(NETFILE *stream) {
+long ftell_net(NETFILE *stream) {
+	return(ftello_net(stream));
+}
+
+off_t flength_net(NETFILE *stream) {
 	if (!stream) {
 		return(-1);
 	}
